@@ -650,6 +650,7 @@ static hci_map ver_map[] = {
 	{ "3.0",	0x05 },
 	{ "4.0",	0x06 },
 	{ "4.1",	0x07 },
+	{ "4.2",	0x08 },
 	{ NULL }
 };
 
@@ -899,8 +900,15 @@ static int __same_bdaddr(int dd, int dev_id, long arg)
 
 int hci_get_route(bdaddr_t *bdaddr)
 {
-	return hci_for_each_dev(HCI_UP, __other_bdaddr,
+	int dev_id;
+
+	dev_id = hci_for_each_dev(HCI_UP, __other_bdaddr,
 				(long) (bdaddr ? bdaddr : BDADDR_ANY));
+	if (dev_id < 0)
+		dev_id = hci_for_each_dev(HCI_UP, __same_bdaddr,
+				(long) (bdaddr ? bdaddr : BDADDR_ANY));
+
+	return dev_id;
 }
 
 int hci_devid(const char *str)
@@ -1036,6 +1044,12 @@ int hci_open_dev(int dev_id)
 	struct sockaddr_hci a;
 	int dd, err;
 
+	/* Check for valid device id */
+	if (dev_id < 0) {
+		errno = ENODEV;
+		return -1;
+	}
+
 	/* Create HCI socket */
 	dd = socket(AF_BLUETOOTH, SOCK_RAW | SOCK_CLOEXEC, BTPROTO_HCI);
 	if (dd < 0)
