@@ -29,6 +29,7 @@
 
 #include <stdlib.h>
 #include <stdint.h>
+#include <inttypes.h>
 #include <stdbool.h>
 #include <errno.h>
 #include <unistd.h>
@@ -57,6 +58,7 @@
 #include "src/error.h"
 #include "src/sdpd.h"
 #include "src/dbus-common.h"
+#include "src/shared/util.h"
 
 #include "avctp.h"
 #include "avrcp.h"
@@ -190,6 +192,7 @@ struct pending_list_items {
 	GSList *items;
 	uint32_t start;
 	uint32_t end;
+	uint64_t total;
 };
 
 struct avrcp_player {
@@ -490,11 +493,11 @@ static uint32_t get_company_id(const uint8_t cid[3])
  *
  * Set three-byte Company_ID into outgoing AVRCP message
  */
-static void set_company_id(uint8_t cid[3], const uint32_t cid_in)
+static void set_company_id(uint8_t cid[3], uint32_t cid_in)
 {
-	cid[0] = cid_in >> 16;
-	cid[1] = cid_in >> 8;
-	cid[2] = cid_in;
+	cid[0] = (cid_in & 0xff0000) >> 16;
+	cid[1] = (cid_in & 0x00ff00) >> 8;
+	cid[2] = (cid_in & 0x0000ff);
 }
 
 static const char *attr_to_str(uint8_t attr)
@@ -1055,7 +1058,7 @@ static uint8_t avrcp_handle_get_element_attributes(struct avrcp *session,
 {
 	struct avrcp_player *player = target_get_player(session);
 	uint16_t len = ntohs(pdu->params_len);
-	uint64_t identifier = bt_get_le64(&pdu->params[0]);
+	uint64_t identifier = get_le64(&pdu->params[0]);
 	uint16_t pos;
 	uint8_t nattr;
 	GList *attr_ids;
@@ -1081,7 +1084,7 @@ static uint8_t avrcp_handle_get_element_attributes(struct avrcp *session,
 		for (i = 0, len = 0, attr_ids = NULL; i < nattr; i++) {
 			uint32_t id;
 
-			id = bt_get_be32(&pdu->params[9] + (i * sizeof(id)));
+			id = get_be32(&pdu->params[9] + (i * sizeof(id)));
 
 			/* Don't add invalid attributes */
 			if (id == AVRCP_MEDIA_ATTRIBUTE_ILLEGAL ||
@@ -1989,13 +1992,13 @@ static void avrcp_parse_attribute_list(struct avrcp_player *player,
 		uint32_t id;
 		uint16_t charset, len;
 
-		id = bt_get_be32(&operands[i]);
+		id = get_be32(&operands[i]);
 		i += sizeof(uint32_t);
 
-		charset = bt_get_be16(&operands[i]);
+		charset = get_be16(&operands[i]);
 		i += sizeof(uint16_t);
 
-		len = bt_get_be16(&operands[i]);
+		len = get_be16(&operands[i]);
 		i += sizeof(uint16_t);
 
 		if (charset == 106) {
@@ -2125,9 +2128,9 @@ static struct media_item *parse_media_element(struct avrcp *session,
 	if (len < 13)
 		return NULL;
 
-	uid = bt_get_be64(&operands[0]);
+	uid = get_be64(&operands[0]);
 
-	namelen = MIN(bt_get_be16(&operands[11]), sizeof(name) - 1);
+	namelen = MIN(get_be16(&operands[11]), sizeof(name) - 1);
 	if (namelen > 0) {
 		memcpy(name, &operands[13], namelen);
 		name[namelen] = '\0';
@@ -2160,11 +2163,11 @@ static struct media_item *parse_media_folder(struct avrcp *session,
 	if (len < 12)
 		return NULL;
 
-	uid = bt_get_be64(&operands[0]);
+	uid = get_be64(&operands[0]);
 	type = operands[8];
 	playable = operands[9];
 
-	namelen = MIN(bt_get_be16(&operands[12]), sizeof(name) - 1);
+	namelen = MIN(get_be16(&operands[12]), sizeof(name) - 1);
 	if (namelen > 0) {
 		memcpy(name, &operands[14], namelen);
 		name[namelen] = '\0';
@@ -2189,7 +2192,7 @@ static gboolean avrcp_list_items_rsp(struct avctp *conn, uint8_t *operands,
 	struct avrcp_player *player = session->controller->player;
 	struct pending_list_items *p = player->p;
 	uint16_t count;
-	uint32_t items, total;
+	uint64_t items;
 	size_t i;
 	int err = 0;
 
@@ -2211,7 +2214,7 @@ static gboolean avrcp_list_items_rsp(struct avctp *conn, uint8_t *operands,
 		goto done;
 	}
 
-	count = bt_get_be16(&operands[6]);
+	count = get_be16(&operands[6]);
 	if (count == 0)
 		goto done;
 
@@ -2221,7 +2224,7 @@ static gboolean avrcp_list_items_rsp(struct avctp *conn, uint8_t *operands,
 		uint16_t len;
 
 		type = operands[i++];
-		len = bt_get_be16(&operands[i]);
+		len = get_be16(&operands[i]);
 		i += 2;
 
 		if (type != 0x03 && type != 0x02) {
@@ -2249,9 +2252,12 @@ static gboolean avrcp_list_items_rsp(struct avctp *conn, uint8_t *operands,
 	}
 
 	items = g_slist_length(p->items);
-	total = p->end - p->start;
-	if (items < total) {
-		avrcp_list_items(session, p->start + items + 1, p->end);
+
+	DBG("start %u end %u items %" PRIu64 " total %" PRIu64 "", p->start,
+						p->end, items, p->total);
+
+	if (items < p->total) {
+		avrcp_list_items(session, p->start + items, p->end);
 		return FALSE;
 	}
 
@@ -2282,8 +2288,8 @@ static void avrcp_list_items(struct avrcp *session, uint32_t start,
 
 	pdu->params[0] = player->scope;
 
-	bt_put_be32(start, &pdu->params[1]);
-	bt_put_be32(end, &pdu->params[5]);
+	put_be32(start, &pdu->params[1]);
+	put_be32(end, &pdu->params[5]);
 
 	pdu->params[9] = 1;
 
@@ -2318,7 +2324,7 @@ static gboolean avrcp_change_path_rsp(struct avctp *conn,
 		goto done;
 	}
 
-	ret = bt_get_be32(&pdu->params[1]);
+	ret = get_be32(&pdu->params[1]);
 
 done:
 	if (ret < 0) {
@@ -2353,10 +2359,10 @@ static gboolean avrcp_set_browsed_player_rsp(struct avctp *conn,
 							operand_count < 13)
 		return FALSE;
 
-	player->uid_counter = bt_get_be16(&pdu->params[1]);
+	player->uid_counter = get_be16(&pdu->params[1]);
 	player->browsed = true;
 
-	items = bt_get_be32(&pdu->params[3]);
+	items = get_be32(&pdu->params[3]);
 
 	depth = pdu->params[9];
 
@@ -2448,8 +2454,8 @@ static void avrcp_get_item_attributes(struct avrcp *session, uint64_t uid)
 
 	pdu->pdu_id = AVRCP_GET_ITEM_ATTRIBUTES;
 	pdu->params[0] = 0x03;
-	bt_put_be64(uid, &pdu->params[1]);
-	bt_put_be16(player->uid_counter, &pdu->params[9]);
+	put_be64(uid, &pdu->params[1]);
+	put_be16(player->uid_counter, &pdu->params[9]);
 	pdu->param_len = htons(12);
 
 	avctp_send_browsing_req(session->conn, buf, sizeof(buf),
@@ -2620,6 +2626,7 @@ static int ct_list_items(struct media_player *mp, const char *name,
 	p = g_new0(struct pending_list_items, 1);
 	p->start = start;
 	p->end = end;
+	p->total = (uint64_t) (p->end - p->start) + 1;
 	player->p = p;
 
 	return 0;
@@ -2633,9 +2640,9 @@ static void avrcp_change_path(struct avrcp *session, uint8_t direction,
 	struct avrcp_browsing_header *pdu = (void *) buf;
 
 	memset(buf, 0, sizeof(buf));
-	bt_put_be16(player->uid_counter, &pdu->params[0]);
+	put_be16(player->uid_counter, &pdu->params[0]);
 	pdu->params[2] = direction;
-	bt_put_be64(uid, &pdu->params[3]);
+	put_be64(uid, &pdu->params[3]);
 	pdu->pdu_id = AVRCP_CHANGE_PATH;
 	pdu->param_len = htons(11);
 
@@ -2679,8 +2686,8 @@ static gboolean avrcp_search_rsp(struct avctp *conn, uint8_t *operands,
 		goto done;
 	}
 
-	player->uid_counter = bt_get_be16(&pdu->params[1]);
-	ret = bt_get_be32(&pdu->params[3]);
+	player->uid_counter = get_be16(&pdu->params[1]);
+	ret = get_be32(&pdu->params[3]);
 
 done:
 	media_player_search_complete(mp, ret);
@@ -2699,8 +2706,8 @@ static void avrcp_search(struct avrcp *session, const char *string)
 	stringlen = strnlen(string, sizeof(buf) - len);
 	len += stringlen;
 
-	bt_put_be16(AVRCP_CHARSET_UTF8, &pdu->params[0]);
-	bt_put_be16(stringlen, &pdu->params[2]);
+	put_be16(AVRCP_CHARSET_UTF8, &pdu->params[0]);
+	put_be16(stringlen, &pdu->params[2]);
 	memcpy(&pdu->params[4], string, stringlen);
 	pdu->pdu_id = AVRCP_SEARCH;
 	pdu->param_len = htons(len - AVRCP_BROWSING_HEADER_LENGTH);
@@ -2737,8 +2744,8 @@ static void avrcp_play_item(struct avrcp *session, uint64_t uid)
 	pdu->packet_type = AVRCP_PACKET_TYPE_SINGLE;
 
 	pdu->params[0] = player->scope;
-	bt_put_be64(uid, &pdu->params[1]);
-	bt_put_be16(player->uid_counter, &pdu->params[9]);
+	put_be64(uid, &pdu->params[1]);
+	put_be16(player->uid_counter, &pdu->params[9]);
 
 	length = AVRCP_HEADER_LENGTH + ntohs(pdu->params_len);
 
@@ -2783,8 +2790,8 @@ static void avrcp_add_to_nowplaying(struct avrcp *session, uint64_t uid)
 	pdu->packet_type = AVRCP_PACKET_TYPE_SINGLE;
 
 	pdu->params[0] = player->scope;
-	bt_put_be64(uid, &pdu->params[1]);
-	bt_put_be16(player->uid_counter, &pdu->params[9]);
+	put_be64(uid, &pdu->params[1]);
+	put_be16(player->uid_counter, &pdu->params[9]);
 
 	length = AVRCP_HEADER_LENGTH + ntohs(pdu->params_len);
 
@@ -2893,7 +2900,7 @@ avrcp_parse_media_player_item(struct avrcp *session, uint8_t *operands,
 	if (len < 28)
 		return NULL;
 
-	id = bt_get_be16(&operands[0]);
+	id = get_be16(&operands[0]);
 
 	player = find_ct_player(session, id);
 	if (player == NULL) {
@@ -2907,7 +2914,7 @@ avrcp_parse_media_player_item(struct avrcp *session, uint8_t *operands,
 
 	media_player_set_type(mp, type_to_string(operands[2]));
 
-	subtype = bt_get_be32(&operands[3]);
+	subtype = get_be32(&operands[3]);
 
 	media_player_set_subtype(mp, subtype_to_string(subtype));
 
@@ -2921,7 +2928,7 @@ avrcp_parse_media_player_item(struct avrcp *session, uint8_t *operands,
 
 	avrcp_player_parse_features(player, &operands[8]);
 
-	namelen = bt_get_be16(&operands[26]);
+	namelen = get_be16(&operands[26]);
 	if (namelen > 0 && namelen + 28 == len) {
 		namelen = MIN(namelen, sizeof(name) - 1);
 		memcpy(name, &operands[28], namelen);
@@ -2981,7 +2988,7 @@ static gboolean avrcp_get_media_player_list_rsp(struct avctp *conn,
 		return FALSE;
 
 	removed = g_slist_copy(session->controller->players);
-	count = bt_get_be16(&operands[6]);
+	count = get_be16(&operands[6]);
 
 	for (i = 8; count && i < operand_count; count--) {
 		struct avrcp_player *player;
@@ -2989,7 +2996,7 @@ static gboolean avrcp_get_media_player_list_rsp(struct avctp *conn,
 		uint16_t len;
 
 		type = operands[i++];
-		len = bt_get_be16(&operands[i]);
+		len = get_be16(&operands[i]);
 		i += 2;
 
 		if (type != 0x01) {
@@ -3071,7 +3078,7 @@ static void avrcp_track_changed(struct avrcp *session,
 {
 	if (session->browsing_id) {
 		struct avrcp_player *player = session->controller->player;
-		player->uid = bt_get_be64(&pdu->params[1]);
+		player->uid = get_be64(&pdu->params[1]);
 		avrcp_get_item_attributes(session, player->uid);
 	} else
 		avrcp_get_element_attributes(session);
@@ -3111,7 +3118,7 @@ static void avrcp_addressed_player_changed(struct avrcp *session,
 						struct avrcp_header *pdu)
 {
 	struct avrcp_player *player = session->controller->player;
-	uint16_t id = bt_get_be16(&pdu->params[1]);
+	uint16_t id = get_be16(&pdu->params[1]);
 
 	if (player != NULL && player->id == id)
 		return;
@@ -3123,7 +3130,7 @@ static void avrcp_addressed_player_changed(struct avrcp *session,
 			return;
 	}
 
-	player->uid_counter = bt_get_be16(&pdu->params[3]);
+	player->uid_counter = get_be16(&pdu->params[3]);
 	session->controller->player = player;
 
 	if (player->features != NULL)
@@ -3136,7 +3143,7 @@ static void avrcp_uids_changed(struct avrcp *session, struct avrcp_header *pdu)
 {
 	struct avrcp_player *player = session->controller->player;
 
-	player->uid_counter = bt_get_be16(&pdu->params[1]);
+	player->uid_counter = get_be16(&pdu->params[1]);
 }
 
 static gboolean avrcp_handle_event(struct avctp *conn,
@@ -3220,7 +3227,8 @@ static gboolean avrcp_get_capabilities_resp(struct avctp *conn,
 	uint16_t events = 0;
 	uint8_t count;
 
-	if (pdu == NULL || pdu->params[0] != CAP_EVENTS_SUPPORTED)
+	if (code == AVC_CTYPE_REJECTED || code == AVC_CTYPE_NOT_IMPLEMENTED ||
+			pdu == NULL || pdu->params[0] != CAP_EVENTS_SUPPORTED)
 		return FALSE;
 
 	/* Connect browsing if pending */
@@ -3602,6 +3610,7 @@ static void state_changed(struct btd_device *device, avctp_state_t old_state,
 		session_init_browsing(session);
 
 		break;
+	case AVCTP_STATE_BROWSING_CONNECTING:
 	default:
 		return;
 	}
