@@ -35,11 +35,10 @@
 #include <syslog.h>
 #include <termios.h>
 #include <time.h>
+#include <poll.h>
 #include <sys/time.h>
-#include <sys/poll.h>
 #include <sys/param.h>
 #include <sys/ioctl.h>
-#include <sys/socket.h>
 #include <sys/uio.h>
 
 #include <bluetooth/bluetooth.h>
